# -*- coding: utf-8 -*-
# Copyright 2016, Red Hat, Inc.
# License: GPL-2.0+ <http://spdx.org/licenses/GPL-2.0+>
# See the LICENSE file for more details on Licensing

'''Unit tests for libtaskotron/minion.py'''

import argparse
import mock
import pytest
import shlex

from libtaskotron import minion
from libtaskotron import main


@pytest.mark.usefixtures('setup')
class TestBaseMinion(object):

    @pytest.fixture
    def setup(self, monkeypatch):
        self.ref_arg_data = {'artifactsdir': '/somedir/',
                             'task': '/path/mytask.yaml',
                             'ssh_privkey': None,
                             '_orig_args': {'item': 'foo',
                                            'type': 'koji_tag'}}
        self.bm = minion.BaseMinion(None, self.ref_arg_data)
        self.bm.taskdir = '/minion/path/'
        self.mock_ssh = mock.Mock()
        self.bm.ssh = self.mock_ssh

    def test_arg_data_exlude_nothing_extra(self):
        '''If there's some option defined in arg_data_exclude that is not present in current
        argparser, it's either a typo or some leftover.'''
        args_names = [action.dest for action in main.get_argparser()._actions]
        for exclude in minion.BaseMinion.arg_data_exclude:
            assert exclude in args_names

    def test_forward_args_allowed_actions(self):
        '''The only allowed actions for non-ignored argparser options are 'store', 'store_true',
        and 'append', otherwise it breaks libtaskotron.python_utils.reverse_argparse()'''
        for action in main.get_argparser()._actions:
            if action.dest in minion.BaseMinion.arg_data_exclude:
                continue
            # also allow HelpAction, that gets added automatically
            assert isinstance(action, (argparse._StoreAction, argparse._StoreTrueAction,
                                       argparse._AppendAction, argparse._HelpAction))

    def test_run_cmd_called(self):
        '''This ensures ssh.cmd() was called with a single argument. Other test_run_* methods rely
        on this, if this ever changes, you need to verify them as well.'''
        self.bm._run()
        cmd_calls = [call for call in self.mock_ssh.method_calls if call[0] == 'cmd']
        assert len(cmd_calls) == 1
        pos_args = cmd_calls[0][1]
        assert len(pos_args) == 1
        cmdline = pos_args[0]
        assert isinstance(cmdline, basestring)

    def test_run_escape(self):
        '''Cmdline arguments must be escaped when running the task on minion'''
        self.ref_arg_data['task'] = 'my sneaky task*; name.yaml'
        self.ref_arg_data['_orig_args']['item'] = 'foo (bar); baz'
        self.bm._run()
        cmd_calls = [call for call in self.mock_ssh.method_calls if call[0] == 'cmd']
        pos_args = cmd_calls[0][1]
        cmdline = pos_args[0]
        elems = shlex.split(cmdline)
        assert self.ref_arg_data['task'] in elems
        assert self.ref_arg_data['_orig_args']['item'] in elems

    def test_run_formula_path(self):
        '''We have to adjust formula path to local directory'''
        self.ref_arg_data['task'] = '/some/host/path/task.yaml'
        self.bm._run()
        cmd_calls = [call for call in self.mock_ssh.method_calls if call[0] == 'cmd']
        pos_args = cmd_calls[0][1]
        cmdline = pos_args[0]
        elems = shlex.split(cmdline)
        assert 'task.yaml' in elems
        assert self.ref_arg_data['task'] not in elems

    def test_run_add_local(self):
        self.bm._run()
        cmd_calls = [call for call in self.mock_ssh.method_calls if call[0] == 'cmd']
        pos_args = cmd_calls[0][1]
        cmdline = pos_args[0]
        elems = shlex.split(cmdline)
        assert '--local' in elems

    def test_run_exclude_args(self):
        self.bm.arg_data_exclude = ['item']
        self.bm._run()
        cmd_calls = [call for call in self.mock_ssh.method_calls if call[0] == 'cmd']
        pos_args = cmd_calls[0][1]
        cmdline = pos_args[0]
        elems = shlex.split(cmdline)
        assert '--item' not in elems
        assert 'foo' not in elems
        assert '--type' in elems
